package Text::Amuse::Compile::Webfonts;

use strict;
use warnings FATAL => 'all';
use utf8;

=head1 NAME

Text::Amuse::Compile::Webfonts - OBSOLETE class to parse and validate webfonts for Text::Amuse::Compile

=head1 SYNOPSIS

This class is not used anymore.

=cut

1;
